function [xmin, xmax, txt] = Read_Adnotare(fname, info);
% Version 2 improved
% Read the information from a annotated file with Praat software 
% (extension *.TextGrid). Keep only the information situted of annotation
% level - "phonem" (if this level exists).
% Info - boolean parameter used to enable/disable the diplaying of
% information readed ( segmentation levels, id, duration etc.)
%--------------------------------------------------------------------------
% Copyright(c) Zbancioc Marius-Dan, 2014, zmarius@etti.tuiasi.ro
% 1)Institute of Computer Science, Roumanian Academy, Iasi Branch, 
% 2)Technical University "Gheorghe Asachi" of Iasi, Romania
%--------------------------------------------------------------------------
% References:
%- M. Zbancioc, M. Feraru, The Analysis of the FCM and WKNN Algorithms Performance for the Emotional Corpus SROL, Advances in Electrical and Computer Engineering, Vol. 12, Issue 3, 2012, pp. 33  38, ISSN: 1582-7445, e-ISSN: 1844-7600, Digital Object Identifier: 10.4316/AECE.2012.03005
%- M. Feraru, M. Zbancioc, Speech Emotion Recognition for SROL Database using Weighted KNN algorithm, International Conference on Electronics, Computers and Artificial Intelligence (ECAI2013), June 27-29, 2013, Pitesti, Romania, ISBN: 978-1-4673-4935-2, Digital Object Identifier: 10.1109/ECAI.2013.6636189 2013 IEEE
%- M. Zbancioc, M. Feraru, A Study about MFCC Relevance in Emotion Classification for SROL Database, Proceedings of 4th International Symposium on Electrical and Electronics Engineering, ISEEE 2013, October, 11-13, 2013, Gala?i, Romnia, ISBN 978-1-4799-2441-7, IEEE Catalog Number CFP1393K-USB.
%- M. Zbancioc, M. Feraru, Using the Lyapunov Exponent from Cepstral Coefficients for Automatic Emotion Recognition, Proc of 8th IEEE International Conference on Electrical and Power Engineering  EPE 2014, 16-17 Oct., Ia?i, Romnia, ISSN: 978-1-4799-5848-1.
%- M. Feraru, M. Zbancioc, Emotion Recognition in Romanian Language using LPC Features, The 4th IEEE International Conference on E-Health and Bioengineering - EHB 2013, November 21-23, 2013, Ia?i, Romania, ISBN: 978-1-4799-2373-1/13/$31.00 2013 IEEE

if nargin<1
    dir_cur=pwd;
    [fname,fpath]=uigetfile('*.txt;*.TextGrid','Fisier adnotat');
    if ~strcmp(dir_cur,fpath(1:end-1)), cd(fpath); end
    fid=fopen(fname);
    cd(dir_cur);
else
    fid=fopen(fname,'r');
end
if nargin<2, info=0; end
% Read each line of the annotated file
% Search the special characters ':' '='
%       size = 7        x = 0.7899
%       item [1]:       intervals [1]:
% The conversion of a character sting into a numeric format is made with:
%   str2num(t(6:end))       sau se poate utiliza functia sscanf:
%   A = ['abc 46 6 ghi'; 'def 7 89 jkl']
%   for k = 1:2, C(k,:) = sscanf(A(k, :)', '%*s %d %d %*s', [1, inf]); end
% values specified with '*/asterisc' are not displayed at the output.


nitems = cauta1(fid,'size');
for i = 1 : nitems
    while(~cauta2(fid,'item',i)), ; end
    string = cauta1(fid,'name','s'); if info, disp(sprintf('Item[%d]: %s',i,string)); end
    %if(strcmp(lower(string),'"sentence"') | strcmp(lower(string),'"propozitie"'))
    if(strcmp(lower(string(1:4)),'"sen') | strcmp(lower(string(1:5)),'"prop'))
        nprop = cauta1(fid,'size');
        xmin = zeros(1,nprop); xmax=xmin; txt = zeros(nprop,6); %"aseara"
        for j = 1 : nprop
            xmin(j) = cauta1(fid,'xmin');
            xmax(j) = cauta1(fid,'xmax');
            string = cauta1(fid,'text','s'); % neglect/extract "
            poz = find(string=='"'); if ~isempty(poz), string=string(poz(1)+1:poz(2)-1); end
            if length(string)<=6,
                txt(j,1:length(string)) = string;
            else
                txt(j,:) = string(1:6);
            end
%            disp(j); disp(char(txt(j,:)));
        end
%        disp('gata');
        
    end
end
if info, disp(char(txt')); end

%    switch tline(1:poz1-2)
%        case 'size'
%            nitems = str2num(tline(poz1+1:end));

fclose(fid);

%-----------------------------------------------------------------------
function n = cauta1 (fid, searched_string, type);
% The function use the symbol/character '=' (ex:  'intervals size = 109')
% After this symbol will have an identificator/class name or a numerical value

if nargin<3, type=0; end    %numeric
n=0;
while 1
    tline = fgetl(fid);     % read each line of the file
    if ~ischar(tline), break, end

    poz = findstr(tline,'='); %poz = find(tline == '=');
    if isempty(poz), continue; end
    lastspace = findstr(tline(poz-2:-1:1),' ');  %lastspace = find(tline(poz-2:-1:1) == ' '); 
    if isempty(lastspace), lastspace=1; else lastspace = poz-lastspace(1); end
    if strcmp(tline(lastspace:poz-2),searched_string)
        if ~type
            n = str2num(tline(poz+1:end)); return;
        else
            n = tline(poz+1:end); 
            n = n(find(~isspace(n))); return;    %neglect the space char
        end
    end
end
%-----------------------------------------------------------------------
function n = cauta2 (fid, string, number);
% The function use character ':' (ex:  'interval [5]:')
% Will return after a comparison of the the number plased between paranthesis, or TRUE,FALSE 
% if it was specified the 3-th parameter (the searched numeric value) 

n=0;
while 1
    tline = fgetl(fid);     % read each line of the file
    if ~ischar(tline), break, end
       
    poz = findstr(tline,':'); %poz = find(tline == ':');
    if isempty(poz), continue; end
    space = findstr(tline(poz-1:-1:1), ' '); %space = find(tline(poz-1:-1:1) == ' ');
    if isempty(space), ch2=1, else ch2=poz-space(1)-1; end
    if length(space)<2, ch1=1; else ch1=poz-space(2)+1; end
    if strcmp(tline(ch1:ch2),string)
        n = sscanf(tline,'%*s [%d] %*s'); if isempty(n), n=0; end
        %if ch2+3<=poz-2, n = str2num(tline(ch2+3:poz-2)); end
        if nargin == 3,
            n = (n==number); return %veriffy 1=TRUE (n=number)
        end
    end
end

%while 1
%    tline = fgetl(fid); 
%    if ~ischar(tline), break, end
%    disp(tline)
%end