function [t, vFk] = Read_F0_Praat(fname,fpath,plot_on);
% Read the F0 fundamental frequency values respectivelly of the formants
% from files generated by Praat software
% plot_on = 0 (disable) do not plot any graphic
%         = 1 (enable) display the readed values and plot the graphics
%--------------------------------------------------------------------------
% Copyright(c) Zbancioc Marius-Dan, 2014, zmarius@etti.tuiasi.ro
% 1)Institute of Computer Science, Roumanian Academy, Iasi Branch, 
% 2)Technical University "Gheorghe Asachi" of Iasi, Romania
%--------------------------------------------------------------------------
% References:
%- M. Zbancioc, M. Feraru, The Analysis of the FCM and WKNN Algorithms Performance for the Emotional Corpus SROL, Advances in Electrical and Computer Engineering, Vol. 12, Issue 3, 2012, pp. 33  38, ISSN: 1582-7445, e-ISSN: 1844-7600, Digital Object Identifier: 10.4316/AECE.2012.03005
%- M. Feraru, M. Zbancioc, Speech Emotion Recognition for SROL Database using Weighted KNN algorithm, International Conference on Electronics, Computers and Artificial Intelligence (ECAI2013), June 27-29, 2013, Pitesti, Romania, ISBN: 978-1-4673-4935-2, Digital Object Identifier: 10.1109/ECAI.2013.6636189 2013 IEEE
%- M. Zbancioc, M. Feraru, A Study about MFCC Relevance in Emotion Classification for SROL Database, Proceedings of 4th International Symposium on Electrical and Electronics Engineering, ISEEE 2013, October, 11-13, 2013, Gala?i, Romnia, ISBN 978-1-4799-2441-7, IEEE Catalog Number CFP1393K-USB.
%- M. Zbancioc, M. Feraru, Using the Lyapunov Exponent from Cepstral Coefficients for Automatic Emotion Recognition, Proc of 8th IEEE International Conference on Electrical and Power Engineering  EPE 2014, 16-17 Oct., Ia?i, Romnia, ISSN: 978-1-4799-5848-1.
%- M. Feraru, M. Zbancioc, Emotion Recognition in Romanian Language using LPC Features, The 4th IEEE International Conference on E-Health and Bioengineering - EHB 2013, November 21-23, 2013, Ia?i, Romania, ISBN: 978-1-4799-2373-1/13/$31.00 2013 IEEE

t=''; vFk='';
if nargin<1
    dir_cur=pwd;
    [fname,fpath]=uigetfile('*.F0Praat; *.FkPraat','Open Praat output files');
    if ~strcmp(dir_cur,fpath(1:end-1))
        cd(fpath);
    end
end
if nargin>=2, fname = [fpath fname]; end
if nargin<3; plot_on=0; end
dir_cur=pwd;

fid = fopen(fname,'r');
if fid==-1, disp(strcat('Eroare la deschidere fisier ', fname, '.')); return; end

nlin=0;
while(1)
    text = fgets(fid);
    if text==-1,
        break;
    else
        nlin=nlin+1;
    end
end
fseek(fid,0,-1); %fseek(fid,0,'bof'); %la inceput fisier
text = fgets(fid); %jump after the firs line
[t,ncol] = sscanf(text,'%s ',inf);

s = zeros(nlin, ncol);
for i=1:nlin-1
    for j=1:ncol
        t = fscanf(fid,'%s',1);
        if ~strcmp(t,'--undefined--'), s(i,j)=str2num(t); end
    end
end
fclose(fid);
if plot_on, figure(1); 
    for i=2:ncol, plot(s(:,1),s(:,i)); hold on; end
    hold off; grid; axis tight; xlabel('time'); title('Praat values');
end
t = s(:,1); vFk = s(:,2:ncol);
cd(dir_cur);