function [DB DB_all] = construct_DB_all_emotions
% Build the Data Base from the Emotional Corpus of SROL (Sounds of
% Roumanian Language). If the DB.mat file exists (was previously
% generated), then we will be created DB_all (with 8 sub-structure), which
% is a restructuration of DB acording to the expressed emotions: joy, fury,
% sadness and neutral tone, and according to the gender of the speaker:
% male/female.
% This structure of data DB_all is used by the application which classify
% the emotions (using the classifiers KNN/SVM/FCM), respectivelly for the
% generation/visualisation of the vowels triangle.
%--------------------------------------------------------------------------
% Copyright(c) Zbancioc Marius-Dan, 2014, zmarius@etti.tuiasi.ro
% 1)Institute of Computer Science, Roumanian Academy, Iasi Branch, 
% 2)Technical University "Gheorghe Asachi" of Iasi, Romania
%--------------------------------------------------------------------------
% References:
%- M. Zbancioc, M. Feraru, The Analysis of the FCM and WKNN Algorithms Performance for the Emotional Corpus SROL, Advances in Electrical and Computer Engineering, Vol. 12, Issue 3, 2012, pp. 33  38, ISSN: 1582-7445, e-ISSN: 1844-7600, Digital Object Identifier: 10.4316/AECE.2012.03005
%- M. Feraru, M. Zbancioc, Speech Emotion Recognition for SROL Database using Weighted KNN algorithm, International Conference on Electronics, Computers and Artificial Intelligence (ECAI2013), June 27-29, 2013, Pitesti, Romania, ISBN: 978-1-4673-4935-2, Digital Object Identifier: 10.1109/ECAI.2013.6636189 2013 IEEE
%- M. Zbancioc, M. Feraru, A Study about MFCC Relevance in Emotion Classification for SROL Database, Proceedings of 4th International Symposium on Electrical and Electronics Engineering, ISEEE 2013, October, 11-13, 2013, Gala?i, Romnia, ISBN 978-1-4799-2441-7, IEEE Catalog Number CFP1393K-USB.
%- M. Zbancioc, M. Feraru, Using the Lyapunov Exponent from Cepstral Coefficients for Automatic Emotion Recognition, Proc of 8th IEEE International Conference on Electrical and Power Engineering  EPE 2014, 16-17 Oct., Ia?i, Romnia, ISSN: 978-1-4799-5848-1.
%- M. Feraru, M. Zbancioc, Emotion Recognition in Romanian Language using LPC Features, The 4th IEEE International Conference on E-Health and Bioengineering - EHB 2013, November 21-23, 2013, Ia?i, Romania, ISBN: 978-1-4799-2373-1/13/$31.00 2013 IEEE

if exist('DB.mat','file')==2,
    load DB
else    
    % emotional state: joy,fury,sadness and neutral tone; gender:male/female
    path = {'E:\IPA_sounds_selection\DB100\bucurie\feminin\',...
            'E:\IPA_sounds_selection\DB100\bucurie\masculin\',...
            'E:\IPA_sounds_selection\DB100\furie\feminin\',...
            'E:\IPA_sounds_selection\DB100\furie\masculin\',...
            'E:\IPA_sounds_selection\DB100\tristete\feminin\',...
            'E:\IPA_sounds_selection\DB100\tristete\masculin\',...
            'E:\IPA_sounds_selection\DB100\neutru\feminin\',...
            'E:\IPA_sounds_selection\DB100\neutru\masculin\'};
    emotion_ID=char(zeros(8,2));
    for i=1:8
        %DB{i} = extract_fonem_parameters(path{i});
        DB{i} = extract_phrase_parameters(path{i});
        poz_backslash=find(path{i}=='\'); emotion_ID(i,:)=[path{i}(poz_backslash(end-2)+1) path{i}(poz_backslash(end-1)+1)];
    end
    emotion_ID=upper(emotion_ID);
    emotion_ID(find(emotion_ID=='B'))='J';
    emotion_ID(find(emotion_ID=='T'))='S';
    save DB;
end

DB_all=DB{1};
for i=1:8   %all states(1:8), only female gender 1:2:8, only male 2:2:8
    for k=1:length(DB{i}),  %all the vowels
        for j=1:length(DB_all),
            if strcmp(DB{i}{k}{1}, DB_all{j}{1}),
                DB_all{j}{2}=DB_all{j}{2}+DB{i}{k}{2};
                for m=3:length(DB{i}{k})
                    DB_all{j}{m} = [DB_all{j}{m} DB{i}{k}{m}];
end;end;end;end;end
