function DB = extract_phrase_parameters(path1, path2)
% -------------------------------------------------------------------------
% Variant 5 (added with processing of Lyapunov coef. extracted from mel energies)
% -------------------------------------------------------------------------
% Search the annotation files and select the segmentation level 'phonem' 
% For each phonem will compute a set of parameters:
% - F0 fundamental frequency values (with corections)
% - F1-F4 formants values (mean, std, median)
% - MFCC si LPCC cepstral coefficient values (+ derivate coefficients)
% - energies from mel-frequency bands 
% - nelinear dinamic coef. Lyapunov (for all cepstral coef. and mel energies)
% The enable/disable of the parameters computing is made by the boolean
% variabile: compute_MFCC = 1 (On/Enable) compute_MFCC = 0 (Off/Disable)
%--------------------------------------------------------------------------
% Copyright(c) Zbancioc Marius-Dan, 2014, zmarius@etti.tuiasi.ro
% 1)Institute of Computer Science, Roumanian Academy, Iasi Branch, 
% 2)Technical University "Gheorghe Asachi" of Iasi, Romania
%--------------------------------------------------------------------------
% References:
%- M. Zbancioc, M. Feraru, The Analysis of the FCM and WKNN Algorithms Performance for the Emotional Corpus SROL, Advances in Electrical and Computer Engineering, Vol. 12, Issue 3, 2012, pp. 33  38, ISSN: 1582-7445, e-ISSN: 1844-7600, Digital Object Identifier: 10.4316/AECE.2012.03005
%- M. Feraru, M. Zbancioc, Speech Emotion Recognition for SROL Database using Weighted KNN algorithm, International Conference on Electronics, Computers and Artificial Intelligence (ECAI2013), June 27-29, 2013, Pitesti, Romania, ISBN: 978-1-4673-4935-2, Digital Object Identifier: 10.1109/ECAI.2013.6636189 2013 IEEE
%- M. Zbancioc, M. Feraru, A Study about MFCC Relevance in Emotion Classification for SROL Database, Proceedings of 4th International Symposium on Electrical and Electronics Engineering, ISEEE 2013, October, 11-13, 2013, Gala?i, Romnia, ISBN 978-1-4799-2441-7, IEEE Catalog Number CFP1393K-USB.
%- M. Zbancioc, M. Feraru, Using the Lyapunov Exponent from Cepstral Coefficients for Automatic Emotion Recognition, Proc of 8th IEEE International Conference on Electrical and Power Engineering  EPE 2014, 16-17 Oct., Ia?i, Romnia, ISSN: 978-1-4799-5848-1.
%- M. Feraru, M. Zbancioc, Emotion Recognition in Romanian Language using LPC Features, The 4th IEEE International Conference on E-Health and Bioengineering - EHB 2013, November 21-23, 2013, Ia?i, Romania, ISBN: 978-1-4799-2373-1/13/$31.00 2013 IEEE

dir_cur=pwd;
% emotional state: joy,fury,sadness and neutral tone; gender:male/female
if nargin<1,
%    path1 = 'E:\IPA_sounds_selection\DB\bucurie\feminin\';  %nume_v1.TextGrid annotation director 
    path1 = 'E:\IPA_sounds_selection\DB100\tristete\masculin\';  %nume_v1.TextGrid annotation director 
    %path2 = 'E:\IPA_sounds_selection\DB\bucurie\feminin\';  %nume_[16b]_v12.wav sound files director 
end
if nargin<2,
    path2=path1;
end    
poz_backslash=find(path1=='\'); emotion_ID=[path1(poz_backslash(end-2)+1) path1(poz_backslash(end-1)+1)]; emotion_ID=upper(emotion_ID); clear poz_backslash;
cd(path1);

tip_s=0;
if(emotion_ID(2) == 'F'), tip_s=1; end
if(emotion_ID(2) == 'M'), tip_s=2; end
compute_jitter = 1;
compute_MFCC = 1;
compute_LPCC = 1;
extended_LPCC = 1;
save_cepstral_files = 0;
save_energy_files = 0;
add_lyap_MFCC_LPCC = 1;
add_lyap_energy = 1;
add_energy = 1;

if save_cepstral_files == 1
    if exist([path1 'MFCC\'],'dir')~=7, mkdir([path1 'MFCC\']); end
    if exist([path1 'LPCC\'],'dir')~=7, mkdir([path1 'LPCC\']); end
end
if save_energy_files == 1
    if exist([path1 'Energy\'],'dir')~=7, mkdir([path1 'Energy\']); end
end

F0_all_values=0; F0_left_values=0; F0_NaN_values=0;
F1_all_values=0; F1_left_values=0; F1_err_values=0; 
nr_prop=0; F1_NaN_values=0; foneme_cu_salturi=0;

fid=fopen(['propozitii_all_' emotion_ID '.tsv'],'w');
% LIST is a array of structures with the fields:
%       name        date        bytes       isdir
list=dir('*.TextGrid');
nr_fis=length(list);

DB={}; lenDB=0;
cd(dir_cur); clear dir_cur;
for i=1:nr_fis
%    disp(strcat(sprintf('%3.1f',i/nr_fis*100),'%')); %display the procent of processed information
    name = list(i).name;
    fprintf(fid,'__________________________________________\r\n%s\r\n',name);
    name_adn = name;
    [xmin xmax txt] = Read_Adnotare([path1 name_adn]);
    poz_pct=find(list(i).name=='.'); poz_pct=poz_pct(end); 
    poz_underline=find(list(i).name=='_'); poz_underline=poz_underline(end); 
%    name_wav=[path2 name(1:poz_pct-6) '[' name(poz_pct-4:poz_pct-2) '].wav'];
%    name_wav = [path2 name(1:poz_underline) '[16b]_' name(poz_underline+1:poz_pct) 'wav'];
    name_wav= [path2 name(1:poz_pct-1) '.wav'];
    
    error=0;
    if ~exist(name_wav, 'file'), 
        errordlg(strcat('EROARE! Fisierul de sunet ', name_wav, ' nu a fost gasit...'),'Error');          
        disp(strcat('EROARE! Fisierul de sunet ', name_wav, ' nu a fost gasit...')); error=1; 
    else
        [x,fs,nbits]=wavread(name_wav);
        len_x=length(x);
        if (len_x/fs)~=xmax(end), 
            errordlg(sprintf('EROARE! Fisierul %s nu are lungime corespunzatoare cu cea din fisierul adnotat', name),'Error');
            disp(sprintf('EROARE! Fisierul %s nu are lungime corespunzatoare cu cea din fisierul adnotat', name)); error=1; 
        end
    end
    
    procesare_energie=0;
    if procesare_energie==1 & error==0
        N=1024; step=10;
        energy = time_energy(x,fs,N,step);
        energy = energy/max(energy);
        e_band = percent_spectral_band_energy(x,fs,N,step);
        for j=1:length(txt)
            start_x = round(xmin(j)*fs); if start_x==0, start_x=1; end
            end_x = round(xmax(j)*fs); if end_x>len_x, end_x=len_x; end
            z = zcr(x(start_x:end_x), fs);

            start_iter = ceil(start_x/step); end_iter = floor((end_x-N)/step);
            % !!! if the lengh of phonem < N window dim. (end_x-start_x)<N
            if end_iter<start_iter; end_iter=start_iter; end

    %        disp(char(txt(j,:)));
            [avg_e, std_e] = fuzzy_mean(energy(start_iter:end_iter));% ,fs,5);
            n_band = size(e_band,1); avg_band = zeros(1,n_band);
            for k=1:n_band
                avg_band(k)=fuzzy_mean(e_band(k,start_iter:end_iter));% ,fs,5);
            end
            class_id = class_name(char(txt(j,:)));
    %       if class_id & avg_e~=0 & ~isnan(std_e),
                if(txt(j,1)==0),
                     fprintf(fid,'_  ,');  % character'_' for pauses between utterances
                else fprintf(fid,'%-3s,',char(txt(j,:)));
                end
                fprintf(fid,'%d,%.2f,%f,%f,',class_id, z, avg_e, std_e);
                fprintf(fid,'%f,', avg_band);
                fprintf(fid,'%d%s\n',j,name(length(path2)+1:end));
    %       end
        end 
    end
    poz_id = find(name(1:poz_underline-1)<'0'|name(1:poz_underline-1)>'9'); poz_id = poz_id(end);
    name_f0 = [path1 'F0_' name(1:poz_id) '_' name(poz_id+1:poz_underline-1)];
    name_fk = [path1 'F1-F4_' name(1:poz_id) '_' name(poz_id+1:poz_underline-1)];
    name_puls = [path1 'Puls_' name(1:poz_id) '_' name(poz_id+1:poz_underline-1)];
    if ~exist(name_f0, 'file'),
        errordlg(strcat('EROARE! Fisierul cu valori F0: ', name_f0, ' nu a fost gasit...'),'Error');          
        disp(strcat('EROARE! Fisierul cu valori F0: ', name_f0, ' nu a fost gasit...')); continue; 
    end
    if ~exist(name_fk, 'file'), 
        errordlg(strcat('EROARE! Fisierul cu valori F1-F4 ', name_fk, ' nu a fost gasit...'),'Error');          
        disp(strcat('EROARE! Fisierul cu valori F1-F4 ', name_fk, ' nu a fost gasit...')); continue; 
    end
    if ~exist(name_puls, 'file'), 
        errordlg(strcat('EROARE! Fisierul cu valori puls ', name_fk, ' nu a fost gasit...'),'Error');          
        disp(strcat('EROARE! Fisierul cu valori F1-puls ', name_fk, ' nu a fost gasit...')); continue; 
    end       
    [tF0, vF0] = Read_F0_Praat(name_f0);
    [tFk, vFk] = Read_F0_Praat(name_fk);
    vPuls = Read_F0_Praat(name_puls);
    %disp(['Window step ' num2str(tF0(2)-tF0(1)) ' ' num2str(tFk(2)-tFk(1)) ' ' name]); %time step
%    fprintf('%d %s', size(txt,1), list(i).name);
    
    for j=1:size(txt,1)
        if sum(txt(j,:))==0, continue; end  %pauses between utterances          
        
        poz_prop = find(tF0>=xmin(j) & tF0<=xmax(j)); 
        vF0_prop = vF0(poz_prop); tF0_prop = tF0(poz_prop);

        poz_prop = find(tFk>=xmin(j) & tFk<=xmax(j));
        vFk_prop = vFk(poz_prop, :);
        
%---------------------------Correction of F0 values ----------------------------
        [vF0c_prop, valid] = Corectie_F0(vF0_prop,tip_s);
        tF0_prop = tF0_prop(valid);
        vF0c_prop = vF0_prop;
        
%---------------------------Find jumps in F1 values ----------------------------
        if sum(txt(j,:))~=0 % length(find(char(txt(j,:))~=0))>0, %daca nu e pauza intre rostiri
            tFk_prop = tFk(poz_prop); clear poz_prop;

            [pozs, pozn] = gaseste_salturi_F1(vFk_prop, 0.3);
            if ~isempty(pozs),
                foneme_cu_salturi = foneme_cu_salturi+1;
%                disp([sprintf('%s %-3s %d | ', name(1:poz_underline-1), char(txt(j,:)), j) sprintf('%g ', tFk_prop(pozs))]);
            end

            while ~isempty(pozs),
                tFk_prop = tFk_prop(pozn); vFk_prop = vFk_prop(pozn, :);
                [pozs, pozn] = gaseste_salturi_F1(vFk_prop, 0.3);
            end
%---------------------------Correction of F1 values ----------------------------
            len_vFk = length(vFk_prop(:,1));
            F1_all_values = F1_all_values+len_vFk;
                                
            [vFk_prop, valid] = Corectie_F1(vFk_prop, txt(j,:), tip_s); tFk_prop = tFk_prop(valid);
            F1_err_values = F1_err_values+len_vFk-length(vFk_prop(:,1));
%            if len_vFk~=length(vFk_prop(:,1)), disp(sprintf('%s %-3s Nr.fonem:%d Nr.cazuri:%d', name(1:poz_underline-1), char(txt(j,:)), j, len_vFk-length(vFk_prop(:,1)) )); end

            % eliminate the F1 values are smaller than < mean*120%
            if ~isempty(vF0c_prop), 
                [vFk_prop, valid] = Corectie_F1_F0(vFk_prop, mean_NaN(vF0c_prop), 1.2); tFk_prop = tFk_prop(valid);
            end
            % eliminate the F1 values where the mean is not in the interval [mediana*50%, mediana*150%]
            if ~isempty(vFk_prop), 
                if (mean(vFk_prop(:,1))<median(vFk_prop(:,1))*0.5) | (mean(vFk_prop(:,1))>median(vFk_prop(:,1))*1.5), 
                    vFk_prop=[];
                end
            else
%            disp(sprintf('%s %-3s Nr.fonem:%d',name(1:poz_underline-1), char(txt(j,:)), j));
                F1_NaN_values=F1_NaN_values+1;
            end
            F1_left_values = F1_left_values+length(vFk_prop(:,1));
            nr_prop=nr_prop+1;
        end
%------------------------ end of F1 Correction ------------------------

        if(txt(j,1)==0),
        %    fprintf(fid,'_  ,');  % character'_' for pauses between utterances
            fprintf(fid,'_  \t');
        else %fprintf(fid,'%-3s,',char(txt(j,:)));
            fprintf(fid,'%-3s\t',char(txt(j,:)));
            F0_all_values = F0_all_values + length(vF0_prop);
            F0_NaN_values = F0_NaN_values + length(find(~vF0_prop)); %length(find(vF0_prop==0));
            F0_left_values = F0_left_values + length(vF0c_prop);
        end
%------------------------ compute jitter and shimmer ------------------------

        if compute_jitter ==1,
            jitter1 = jitter_v1(vF0c_prop);
            jitter2 = jitter_v2(vPuls, xmin(j), xmax(j));
            shimmer = shimmer_v2(x, fs, vPuls, xmin(j), xmax(j));
        else
            jitter1=NaN; jitter2=NaN; shimmer=NaN;
        end
 
%-------------- compute cepstral MFCC or LPCC coef.-----------------------
        if compute_MFCC ==1 || compute_LPCC==1,
            t = [0:len_x]/fs;
            vPuls_prop = vPuls(find(vPuls >=xmin(j) & vPuls <=xmax(j))); 
            if length(vPuls_prop)<=1, vMFCC=[]; vLPCC=[]; vEnergy=[];
            else
                vPuls_start = vPuls_prop(1);
                vMFCC = []; vLPCC=[]; vEnergy=[];
                % compute cepstral coef. only from vowel areas (where the puls values F0>=70Hz)
                for k = 2 : length(vPuls_prop)
                    if (vPuls_prop(k) - vPuls_prop(k-1))>=1/70 %& vPuls_prop(k) - vPuls_prop(k-1))<=1/500
                        vPuls_stop = vPuls_prop(k-1);
                        if compute_MFCC ==1,
                            [VQ, Energy] = extract_MFCC( x(t>=vPuls_start & t<=vPuls_stop) ,fs);
                            vMFCC = [vMFCC; VQ];
                            Energy = extract_mel_energy( x(t>=vPuls_start & t<=vPuls_stop) ,fs);
                            vEnergy = [vEnergy; Energy];
                        end
                        if compute_LPCC ==1,
                            %[LPCC, A ,E, K, G, AC] = extract_LPCC(x(t>=vPuls_start & t<=vPuls_stop), fs, round(25.6/1000*fs), round(10/1000*fs), nac, nlpcc, plot_on);
                            [LPCC, A ,E, K, G, AC] = extract_LPCC(x(t>=vPuls_start & t<=vPuls_stop), fs, round(25.6/1000*fs));
                            if extended_LPCC
                                vLPCC = [vLPCC; [LPCC, A ,E, K', G, AC]];
                            else
                                vLPCC = [vLPCC; LPCC];
                            end
                        end  
                        vPuls_start = vPuls_prop(k);
                    end
                end
                vPuls_stop = vPuls_prop(end);
                if vPuls_start ~= vPuls_stop
                    if compute_MFCC ==1,
                        [VQ, Energy] = extract_MFCC( x(t>=vPuls_start & t<=vPuls_stop) ,fs);
                        vMFCC = [vMFCC; VQ];
                        Energy = extract_mel_energy( x(t>=vPuls_start & t<=vPuls_stop) ,fs);
                        vEnergy = [vEnergy; Energy];
                    end
                    if compute_LPCC ==1,
                        [LPCC, A ,E, K, G, AC] = extract_LPCC(x(t>=vPuls_start & t<=vPuls_stop), fs, round(25.6/1000*fs));
                        if extended_LPCC
                            vLPCC = [vLPCC; [LPCC, A ,E, K', G, AC]];
                        else
                            vLPCC = [vLPCC; LPCC];
                        end
                    end
                end
            end
        else
            vMFCC=[]; vLPCC=[];
        end
        if save_cepstral_files == 1,
            fid_mfcc = fopen([path1 'MFCC\' name(1:poz_pct-1) '_P' num2str(j) '.mfcc'],'w');
            for k = 1 : size(vMFCC,1)
                fprintf(fid_mfcc ,'%g\t', vMFCC(k,:));
                fprintf(fid_mfcc ,'\r\n');
            end
            fclose(fid_mfcc );
            fid_lpcc = fopen([path1 'LPCC\' name(1:poz_pct-1) '_P' num2str(j) '.lpcc'],'w');
            for k = 1 : size(vLPCC,1)
                fprintf(fid_lpcc,'%g\t', vLPCC(k,:));
                fprintf(fid_lpcc,'\r\n');
            end
            fclose(fid_lpcc);
            save([path1 'MFCC\' name(1:poz_pct-1) '_P' num2str(j) '.mat'], 'vMFCC');
            save([path1 'LPCC\' name(1:poz_pct-1) '_P' num2str(j) '.mat'], 'vLPCC');
        end
        if save_energy_files == 1,
            fid_energy = fopen([path1 'Energy\' name(1:poz_pct-1) '_P' num2str(j) '.energy'],'w');
            for k = 1 : size(vEnergy,1)
                fprintf(fid_energy ,'%g\t', vEnergy(k,:));
                fprintf(fid_energy ,'\r\n');
            end
            fclose(fid_energy);
            save([path1 'Energy\' name(1:poz_pct-1) '_P' num2str(j) '.mat'], 'vEnergy');
        end
        % keep the average values from all analisis windows of 25.6ms and a step of 10ms for each sentence
        if compute_MFCC==1, 
            
            if length(vMFCC)==0, vMFCC = ones(1,48)*NaN;
            else
                VQ = zeros(1,48);
                for k=1:48
                    sel = find(vMFCC(:,k)~=0);
                    if length(sel)==0,
                        VQ(k) = NaN;
                    else
                        VQ(k) = mean(vMFCC(sel,k));
                    end
                end
                %neglect the values of deltaMFCC 80ms and introduce dev_std MFCC
                VQ = [VQ(1:12) zeros(1,12) VQ(13:24) VQ(37:48)];
                for k=1:12
                    sel = find(vMFCC(:,k)~=0);
                    if length(sel)==0,
                        VQ(12+k) = NaN;
                    else
                        VQ(12+k) = std(vMFCC(sel,k));
                    end
                end
                vMFCC = VQ;
            end
        end
        if compute_LPCC ==1, 
            if extended_LPCC, no_LPCC=80; else no_LPCC=21; end
            
            if length(vLPCC)==0, vLPCC = ones(1,no_LPCC)*NaN;
            else
                LPCC = zeros(1,no_LPCC);
                for k=1:no_LPCC
                    sel = find(vLPCC(:,k)~=0);
                    if length(sel)==0,
                        LPCC(k) = NaN;
                    else
                        LPCC(k) = mean(vLPCC(sel,k));
                    end
                end
                vLPCC = LPCC;
            end
        end

        if add_lyap_MFCC_LPCC ==1,
            vLLE = load([path1 'MFCC\' name(1:poz_pct-1) '_P' num2str(j) '.mat'], 'lyap');
            if isempty(fieldnames(vLLE)), 
                fprintf('\nEroare... Fisierul %s incomplet nu are lyap', ['MFCC\' name(1:poz_pct-1) '_P' num2str(j) '.mat']); lyap_MFCC = NaN(12,1);
            elseif size(vLLE.lyap,1)~=12,
                fprintf('\nEroare... Fisierul %s cu dimensiune gresita lyap', ['MFCC\' name(1:poz_pct-1) '_P' num2str(j) '.mat']); lyap_MFCC = NaN(12,1);
            else
                lyap_MFCC = vLLE.lyap(:,1);
            end
            
            
            vLLE = load([path1 'LPCC\' name(1:poz_pct-1) '_P' num2str(j) '.mat'], 'lyap');
            if isempty(fieldnames(vLLE)), 
                fprintf('\nEroare... Fisierul %s incomplet nu are lyap', ['LPCC\' name(1:poz_pct-1) '_P' num2str(j) '.mat']); lyap_MFCC = NaN(21,1);
            elseif size(vLLE.lyap,1)~=21,
                fprintf('\nEroare... Fisierul %s cu dimensiune lyap gresit', ['LPCC\' name(1:poz_pct-1) '_P' num2str(j) '.mat']); lyap_MFCC = NaN(21,1);
            else
                lyap_LPCC = vLLE.lyap(:,1);
            end
        end
        if add_lyap_energy ==1,
            vLLE = load([path1 'Energy\' name(1:poz_pct-1) '_P' num2str(j) '.mat'], 'lyap');
            if isempty(fieldnames(vLLE)), 
                fprintf('\nEroare... Fisierul %s incomplet nu are lyap', ['Energy\' name(1:poz_pct-1) '_P' num2str(j) '.mat']); lyap_energy = NaN(12,1);
            elseif size(vLLE.lyap,1)~=12,
                fprintf('\nEroare... Fisierul %s cu dimensiune gresita lyap', ['Energy\' name(1:poz_pct-1) '_P' num2str(j) '.mat']); lyap_energy = NaN(12,1);
            else
                lyap_energy = vLLE.lyap(:,1);
            end
%             %search the valoarea of Lyapunov coef. with embedding dimension and
%             %lag dimension higher than 3
%             load([path1 'Energy\' name(1:poz_pct-1) '_P' num2str(j) '.mat'], 'vLLE');
%             for no_lyap = 1:12
%                 %lyap_energy(no_lyap) = max(max(vLLE(no_lyap,:,:)));
%                 lyap_energy(no_lyap) = max(max(vLLE(no_lyap,3:end,3:end)));
%             end
        end
        if add_energy ==1,
            if length(vEnergy)==0, vEnergy = ones(1,12)*NaN;
            else
                Energy = zeros(1,12);
                for k=1:12
                    sel = find(vEnergy(:,k)~=0);
                    if length(sel)==0,
                        Energy(k) = NaN;
                    else
                        Energy(k) = mean(vEnergy(sel,k));
                    end
                end
                vEnergy = Energy/sum(Energy); %normalizare 
            end
        end
%------------------- save the parameters in DB data base -------------------
        if length(vF0c_prop)==0, 
            mean_vF0=NaN; std_vF0=NaN; median_vF0=NaN;
        else 
            mean_vF0=mean(vF0c_prop); std_vF0=std(vF0c_prop); median_vF0=median(vF0c_prop);
        end
        if length(vFk_prop)==0, 
            mean_vF1=NaN; std_vF1=NaN; median_vF1=NaN;
            mean_vF2=NaN; std_vF2=NaN; median_vF2=NaN;
            mean_vF3=NaN; std_vF3=NaN; median_vF3=NaN;
            mean_vF4=NaN; std_vF4=NaN; median_vF4=NaN;
        else 
            mean_vF1=mean(vFk_prop(:,1)); std_vF1=std(vFk_prop(:,1)); median_vF1=median(vFk_prop(:,1));
            mean_vF2=mean(vFk_prop(:,2)); std_vF2=std(vFk_prop(:,2)); median_vF2=median(vFk_prop(:,2));
            mean_vF3=mean(vFk_prop(:,3)); std_vF3=std(vFk_prop(:,3)); median_vF3=median(vFk_prop(:,3));
            mean_vF4=mean(vFk_prop(:,4)); std_vF4=std(vFk_prop(:,4)); median_vF4=median(vFk_prop(:,4));
        end

        n_param = 17;
        DBnew = {char(txt(j,:)), 1, mean_vF0, std_vF0, median_vF0, ...
                                    mean_vF1, std_vF1, median_vF1, ...
                                    mean_vF2, std_vF2, median_vF2, ...
                                    mean_vF3, std_vF3, median_vF3, ...
                                    mean_vF4, std_vF4, median_vF4};
        if compute_jitter==1, 
            DBnew(n_param+1:n_param+3) = {jitter1, jitter2, shimmer};
            n_param = n_param+3; 
        end
        if compute_MFCC==1,
            for k=1:48, DBnew(n_param+k) = {vMFCC(k)}; end
            n_param = n_param+48; 
        end
        if compute_LPCC ==1, 
            for k=1:no_LPCC, DBnew(n_param+k) = {vLPCC(k)}; end
            n_param = n_param+no_LPCC; 
        end
        if add_lyap_MFCC_LPCC ==1,
            for k=1:12, DBnew(n_param+k) = {lyap_MFCC(k)}; end
            n_param = n_param + 12; 
            for k=1:21, DBnew(n_param+k) = {lyap_LPCC(k)}; end
            n_param = n_param + 21; 
        end
        if add_energy ==1,
            for k=1:12, DBnew(n_param+k) = {vEnergy(k)}; end
            n_param = n_param + 12; 
        end
        if add_lyap_energy ==1,
            for k=1:12, DBnew(n_param+k) = {lyap_energy(k)}; end
            n_param = n_param + 12; 
        end
        DBnew(n_param+1:n_param+2) = {j, i};
                
%         DBnew = {char(txt(j,:)), 1, mean_vF0, std_vF0, median_vF0, ...
%                                     mean_vF1, std_vF1, median_vF1, ...
%                                     mean_vF2, std_vF2, median_vF2, ...
%                                     mean_vF3, std_vF3, median_vF3, ...
%                                     mean_vF4, std_vF4, median_vF4, ...
%                                     jitter1, jitter2, shimmer,...
%                                     vMFCC(1), vMFCC(2), vMFCC(3), vMFCC(4), vMFCC(5), vMFCC(6), vMFCC(7), vMFCC(8), vMFCC(9), vMFCC(10), vMFCC(11), vMFCC(12),...
%                                     vMFCC(13), vMFCC(14), vMFCC(15), vMFCC(16), vMFCC(17), vMFCC(18), vMFCC(19), vMFCC(20), vMFCC(21), vMFCC(22), vMFCC(23), vMFCC(24),...
%                                     vMFCC(25), vMFCC(26), vMFCC(27), vMFCC(28), vMFCC(29), vMFCC(30), vMFCC(31), vMFCC(32), vMFCC(33), vMFCC(34), vMFCC(35), vMFCC(36),...
%                                     vMFCC(37), vMFCC(38), vMFCC(39), vMFCC(40), vMFCC(41), vMFCC(42), vMFCC(43), vMFCC(44), vMFCC(45), vMFCC(46), vMFCC(47), vMFCC(48),...
%                                     j, i};
        %fprintf(fid,'%.1f,', DBnew{3:end-2}); fprintf(fid,'%s\r\n',char(txt(j,:)));
        fprintf(fid,'%.1f\t', DBnew{3:end-2}); fprintf(fid,'\r\n');
         
        gasit=0;
        if lenDB>1, 
            for k = 1:length(DB)
                if strcmp(DB{k}{1}, char(txt(j,:)))==1  %acelasi caracter
                    gasit=1; DB{k}{2}=DB{k}{2}+1; %numar aparitii
                    DB{k}{3}=[DB{k}{3} mean_vF0]; DB{k}{4}=[DB{k}{4} std_vF0]; DB{k}{5}=[DB{k}{5} median_vF0];
                    DB{k}{6}=[DB{k}{6} mean_vF1]; DB{k}{7}=[DB{k}{7} std_vF1]; DB{k}{8}=[DB{k}{8} median_vF1];
                    DB{k}{9}=[DB{k}{9} mean_vF2]; DB{k}{10}=[DB{k}{10} std_vF2]; DB{k}{11}=[DB{k}{11} median_vF2];
                    DB{k}{12}=[DB{k}{12} mean_vF3]; DB{k}{13}=[DB{k}{13} std_vF3]; DB{k}{14}=[DB{k}{14} median_vF3];
                    DB{k}{15}=[DB{k}{15} mean_vF4]; DB{k}{16}=[DB{k}{16} std_vF4]; DB{k}{17}=[DB{k}{17} median_vF4];
                    n_param = 17;
                    if compute_jitter==1, 
                        DB{k}{18}=[DB{k}{18} jitter1]; DB{k}{19}=[DB{k}{19} jitter2]; DB{k}{20}=[DB{k}{20} shimmer];
                        n_param = n_param+3; 
                    end
                    if compute_MFCC==1, 
                        for m=1:48
                            DB{k}{n_param+m} = [DB{k}{n_param+m} vMFCC(m)];
                        end
                        n_param = n_param+48; 
                    end
                    if compute_LPCC==1, 
                        for m=1:no_LPCC
                            DB{k}{n_param+m} = [DB{k}{n_param+m} vLPCC(m)];
                        end
                        n_param = n_param+no_LPCC; 
                    end
                    if add_lyap_MFCC_LPCC ==1,
                        for m=1:12, DB{k}{n_param+m} = [DB{k}{n_param+m} lyap_MFCC(m)]; end
                        n_param = n_param + 12; 
                        for m=1:21, DB{k}{n_param+m} = [DB{k}{n_param+m} lyap_LPCC(m)]; end
                        n_param = n_param + 21; 
                    end
                    if add_energy ==1,
                        for m=1:12, DB{k}{n_param+m} = [DB{k}{n_param+m} vEnergy(m)]; end
                        n_param = n_param + 12; 
                    end
                    if add_lyap_energy ==1,
                        for m=1:12, DB{k}{n_param+m} = [DB{k}{n_param+m} lyap_energy(m)]; end
                        n_param = n_param + 12; 
                    end                        
                    DB{k}{n_param+1}=[DB{k}{n_param+1} j]; DB{k}{n_param+2}=[DB{k}{n_param+2} i]; %pozitie fonem si index fisier
                    break;
                end
            end
        end
        if lenDB==1
            if strcmp(DB{1}, char(txt(j,:)))==1
                gasit=1; DB{2}=DB{2}+1; %number of occurences
                DB{3}=[DB{3} mean_vF0]; DB{4}=[DB{4} std_vF0]; DB{5}=[DB{5} median_vF0];
                DB{6}=[DB{6} mean_vF1]; DB{7}=[DB{7} std_vF1]; DB{8}=[DB{8} median_vF1];
                DB{9}=[DB{9} mean_vF2]; DB{10}=[DB{10} std_vF2]; DB{11}=[DB{11} median_vF2];
                DB{12}=[DB{12} mean_vF3]; DB{13}=[DB{13} std_vF3]; DB{14}=[DB{14} median_vF3];
                DB{15}=[DB{15} mean_vF4]; DB{16}=[DB{16} std_vF4]; DB{17}=[DB{17} median_vF4];
                n_param = 17;
                if compute_jitter==1, 
                    DB{18}=[DB{18} jitter1]; DB{19}=[DB{19} jitter2]; DB{20}=[DB{20} shimmer];
                    n_param = n_param+3;
                end
                if compute_MFCC==1, 
                    for m=1:48
                        DB{n_param+m} = [DB{n_param+m} vMFCC(m)];
                    end
                    n_param = n_param+48;
                end
                if compute_LPCC==1, 
                    for m=1:no_LPCC
                        DB{n_param+m} = [DB{n_param+m} vLPCC(m)];
                    end
                    n_param = n_param+no_LPCC;
                end
                if add_lyap_MFCC_LPCC ==1,
                    for m=1:12, DB{n_param+m} = [DB{n_param+m} lyap_MFCC(m)]; end
                    n_param = n_param + 12;
                    for m=1:21, DB{n_param+m} = [DB{n_param+m} lyap_LPCC(m)]; end
                    n_param = n_param + 21;
                end
                if add_energy ==1,
                    for m=1:12, DB{n_param+m} = [DB{n_param+m} vEnergy(m)]; end
                    n_param = n_param + 12;
                end
                if add_lyap_energy ==1,
                    for m=1:12, DB{n_param+m} = [DB{n_param+m} lyap_energy(m)]; end
                    n_param = n_param + 12;
                end
                DB{n_param+1}=[DB{n_param+1} j]; DB{n_param+2}=[DB{n_param+2} i];
            else
                DB={DB, DBnew };
                gasit=1; lenDB=2;
            end
        end
        if lenDB==0, DB=DBnew; gasit=1; lenDB=1; end
        if gasit==0 , %fonem nou
            DB = [DB, {DBnew} ]; lenDB=lenDB+1;
        end
    end
end

clear mean_vF0 std_vF0 median_vF0 mean_vF1 std_vF1 median_vF1 mean_vF2 std_vF2 median_vF2 mean_vF3 std_vF3 median_vF3 mean_vF4 std_vF4 median_vF4
fclose('all');

n_parameters = length(DB{1})-2;

%----------------- save the informations in files (with distinct columns) -------------------
fid=fopen([path1 'propozitii_' emotion_ID '.tsv'],'w');
fprintf(fid,'propozitie nr_apar\tF0_mean\tF0_std\tF0_med\tF1_mean\tF1_std\tF1_med\tF2_mean\tF2_std\tF2_med\tF3_mean\tF3_std\tF3_med\tF4_mean\tF4_std\tF4_med\tjit_v1\tjit_v2\tshim\r\n');
for k=1:length(DB)
    fprintf(fid,'%s\t%d\t',DB{k}{1}, DB{k}{2}); 
    for i=3: 17, fprintf(fid,'%.1f\t', mean_NaN(DB{k}{i})); end
    for i=18:n_parameters, fprintf(fid,'%.4f\t', mean_NaN(DB{k}{i})); end
    fprintf(fid, '\r\n');
end
%display all the values on columns foar each phonem
for k=1:length(DB)
    fprintf(fid,'\r\n__________________________________________\r\npropozitia:\t%s\r\n',DB{k}{1});
    fprintf(fid,'F0_mean\tF0_std\tF0_med\tF1_mean\tF1_std\tF1_med\tF2_mean\tF2_std\tF2_med\tF3_mean\tF3_std\tF3_med\tF4_mean\tF4_std\tF4_med\tjit_v1\tjit_v2\tshim\tnr_fon\tfile\r\n');
    for i=1:DB{k}{2}   %numar aparitii
        for j=3: 17, fprintf(fid,'%.1f\t', DB{k}{j}(i)); end
        for j=18:n_param-2, fprintf(fid,'%.4f\t', DB{k}{j}(i)); end
        
        poz_underline=find(list(DB{k}{n_param+2}(i)).name=='_'); poz_underline=poz_underline(end); 
        filename = list(DB{k}{n_param+2}(i)).name(1:poz_underline-1);

        fprintf(fid,'%d\t%s\t',DB{k}{n_param+1}(i), filename); 
        fprintf(fid, '\r\n');
        
        poz_speakerID=find(diff(filename>='0'&filename<='9'));poz_speakerID=poz_speakerID(end)+1;
        speaker_ID=str2num(filename(poz_speakerID:end));
        if length(DB{k})==n_param+2, DB{k}{n_param+3} = speaker_ID; %primul vorbitor gasit
        else find_speaker_ID=0;
            for j=1:length(DB{k}{n_param+3}), if speaker_ID==DB{k}{n_param+3}(j), find_speaker_ID=1;break; end; end
            if find_speaker_ID==0, DB{k}{n_param+3} = [DB{k}{n_param+3} speaker_ID]; end
        end
    end
end
fclose('all');
marker = 'ox+*sdv';
color = 'bgrcmyk';
%rgb=[0 0 1; 0 1 0; 1 0 0; 1 1 0; 1 0 1; 0 1 1; 0 0 0];
rgb=[0 0 1; 1 .5 0; 1 0 0; 0 .5 0; .5 0 .5; .5 .5 .5; 0 0 0; 0 1 0; 0 1 1; 1 1 0];
figure(1); hold on; for k=1:length(DB); 
                        plot(DB{k}{6}, DB{k}{9}, '.w'); text(DB{k}{6}, DB{k}{9}, DB{k}{1},'color', rgb(k,:)); %color(k));
                        plot(mean_NaN(DB{k}{6}), mean_NaN(DB{k}{9}), 'o', 'MarkerSize',20, 'LineWidth',2, 'color', rgb(k,:)); %color(k));
                    end; 
           hold off; axis tight; xlabel('F1'); ylabel('F2'); title(['Vowel formant triangle - ' emotion_ID]);

F0_sel_values = F0_all_values - F0_NaN_values - F0_left_values;
disp(sprintf('Emotion: %s,  Numar foneme/vocale: %d',emotion_ID, nr_prop));
disp(sprintf('Total F0 values: %d, NaN: %d, Erroneous F0: %d, Percent: %.1f', F0_all_values, F0_NaN_values, F0_sel_values, F0_sel_values/F0_all_values*100));
disp(sprintf('Total_F1_values: %d, NaN: %d, Erroneous_F1: %d, Err2_F1: %d', F1_all_values, F1_NaN_values, F1_err_values, F1_all_values-F1_err_values-F1_left_values));

%////////////////////////////////////////////////////////////////////////////////////////////////////////////
%//////////////////////////          Function called from main program           ////////////////////////////
%////////////////////////////////////////////////////////////////////////////////////////////////////////////
function sum = jitter_v1(vF0)
% average values of differences between two consecutive perioads 
% T0=1/vF0 [sec] = fs/vF0 [samples]
len =length(vF0);

if len<=1, sum=NaN; return; end

sum=0; sum2=0; n=0;
for i = 2:len
    if vF0(i)~=0 & vF0(i-1)~=0
        sum = sum + abs( 1/vF0(i) - 1/vF0(i-1) );
        sum2 = sum2 + (1/vF0(i) + 1/vF0(i-1))/2;
        n=n+1;
    end
end
%sum = sum/(len-1); %if there are not "breaks" in F0 secvence
%sum = sum/n;
sum =sum/sum2; %(sum/n)/(sum2/n);

%--------------------------------------------------------------------------
function sum = jitter_v2(vPuls, start_t, end_t)
% average values of differences between two consecutive perioads of pitch
vPuls = vPuls(find(vPuls >=start_t & vPuls <=end_t)); 
len =length(vPuls);
if len<=1, sum=NaN; return; end

sum=0; sum2=0; n=0;
for i = 2:len-1
    if (vPuls(i) - vPuls(i-1))<=1/70 & (vPuls(i+1) - vPuls(i))<=1/70 %& (vPuls(i+1) - vPuls(i))>=1/500 & (vPuls(i) - vPuls(i-1))>=1/500
        sum = sum + abs(vPuls(i+1) - 2*vPuls(i) + vPuls(i-1));
        sum2 = sum2  + (vPuls(i+1) - vPuls(i-1))/2; 
        n=n+1;
    end
end
%sum = sum/(len-1); %if the signal do not have "breaks" - pauses between utterances
%sum = sum/n; 
sum = sum/sum2; %(sum/n)/(sum2/n);
%--------------------------------------------------------------------------
% function sum = shimmer(x, fs, start_t, end_t, vF0)
% % compute the differences between the amplitude "peak to peak" of signal in two consecutive periods of pitch
% len_x = length(x);
% len_w = floor(fs/min(vF0));
% 
% start_x = round(start_t*fs - len_w); if start_x==0, start_x=1; end
% end_x = round(end_t*fs + len_w); if end_x>len_x, end_x=len_x; end
% step_x = 
% 
% sum = 0;
% for i = start_x : step_x : end_x

%--------------------------------------------------------------------------
function suma = shimmer_v2(x, fs, vPuls, start_t, end_t)
% compute the differences between the amplitude "peak to peak" of signal in two consecutive periods of pitch
vPuls = vPuls(find(vPuls >=start_t & vPuls <=end_t));   
len =length(vPuls);
if len<=2, suma=NaN; return; end

start_w = floor(vPuls(1:end-1)*fs); end_w = ceil(vPuls(2:end)*fs);
Avv = zeros(1, len-1);
for i = 1:len-1
    w = x(start_w(i):end_w(i));
    Avv(i) = max(w)-min(w);
end
% suma=0;
% for i = 2:len-1
%     suma = suma + abs( 20*log10(Avv(i)/Avv(i-1)) );
% end    
% suma = suma/(len-2);
suma = sum(abs( 20*log10(Avv(2:end)./Avv(1:end-1))))/(len-2);    

%--------------------------------------------------------------------------
%------------------------- Corection of F0 and F1 -------------------------
%--------------------------------------------------------------------------
function [pozs, pozn] = gaseste_salturi_F1(Fk, prag)
if nargin<2, prag=0.5; end
len = size(Fk,1);
pozs=[];
for i = 2: len
    if ( Fk(i,1) > Fk(i-1,1)*(1+prag) ) | ( Fk(i,1) < Fk(i-1,1)*(1-prag) ), pozs = [pozs i-1 i]; end
end 
if isempty(pozs), pozn=[1:len]; return; end
duplicate = find(diff(pozs)==0);
nduplicate = length(duplicate);
coliniare=[];
for k = 1 : nduplicate 
    if (Fk(pozs(duplicate(k)),1)-Fk(pozs(duplicate(k))-1,1)) * (Fk(pozs(duplicate(k))+1,1)-Fk(pozs(duplicate(k)),1))>0,
        coliniare = [coliniare pozs(duplicate(k))]; pozs(duplicate(k))=0; continue; %daca sunt trei puncte coliniare sunt marcate toate 3 pentru stergere
    end
    if ~isempty(coliniare), 
        if coliniare(end) ~= pozs(duplicate(k)-1), pozs(duplicate(k)-1)=0; end
    else
        pozs(duplicate(k)-1)=0; 
    end
    pozs(duplicate(k)+1)=0; 
    if (k < nduplicate & (duplicate(k+1)-duplicate(k))>2) | k==nduplicate 
        pozs(duplicate(k)+2)=0;
    end
end
pozs = pozs(find(pozs)); lens = length(pozs);
pozn = zeros(1, len-lens);
k=1; p=1;
for i = 1: len
    if k<=lens & pozs(k)==i, k=k+1; continue; end
    pozn(p)=i; p=p+1;
end    

%--------------------------------------------------------------------------
function [F0, valid] = Corectie_F0(F0,tip_s)
if nargin<1, tip_s=0; end
if tip_s==0, valid = find(F0>=65 & F0<=450); F0 = F0(valid); end
if tip_s==1, valid = find(F0>=100);          F0 = F0(valid); end   %female
if tip_s==2, valid = find(F0>=75 & F0<=400); F0 = F0(valid); end   %male
%--------------------------------------------------------------------------
function [Fk, valid] = Corectie_F1(Fk, vowel, tip_s)
if nargin<1, tip_s=0; end
valid = [1: size(Fk,1)];
if vowel(1)=='u', 
    if tip_s==1, valid = find(Fk(:,1)>=300 & Fk(:,1)<=1000); end  %female [500 1000]
    if tip_s==2, valid = find(Fk(:,1)>=250 & Fk(:,1)<=800); end   %male [450 800]
else if (vowel(1)=='i' | vowel(1)=='e') & tip_s==2, valid = find(Fk(:,1)<=800); 
    else %if vowel(1)=='a' && (vowel(2)==0 || vowel(2)=='+'), 
            valid = find(Fk(:,1)<=1000); 
        %end
    end
end    
Fk = Fk(valid,:);

function [Fk, valid] = Corectie_F1_F0(Fk, F0, percent)
% F1 formant must have minimum 1.5 of the fundamental frequency F0
valid = find(Fk(:,1)>=percent*F0); Fk = Fk(valid,:);
%--------------------------------------------------------------------------        
function [v, valid] = Elimina_NaN(v)
valid = find(~isnan(v));
v = v(valid);

function [mv, valid] = mean_NaN(v)
valid = find(~isnan(v));
mv = mean(v(valid));

function [sv, valid] = std_NaN(v)
valid = find(~isnan(v));
smv = std(v(valid));
%--------------------------------------------------------------------------
% compute the zero crossing rate in a interval of 1 second
function z = zcr(x, fs)
z = length(find(diff(x>0)))* fs / length(x);
if length(z)==0, z=0; end
%--------------------------------------------------------------------------
function e = time_energy(s,fs,N,step,iter)
len=length(s); 
% when the dimension of analysis window is higher than the signal secvence(len<N)
% estimate (return) the energy for a window N samples
if nargin<3; N=1024; end 
if nargin<4; step=fix(N/2); end 
if len<N; e = sum(s)*N/len; return; end 
if nargin<5 | len < ((iter-1)*step+N),
    iter=floor((len-N)/step) +1;
end

e = zeros(1, iter);
for i=1:step:iter*step
    e(ceil(i/step)) = sum(abs(s(i:i+N-1)).^2);
end
%--------------------------------------------------------------------------
function e_band = percent_spectral_band_energy(s,fs,N,step,iter)
len=length(s); 
% complete the signal with zero when the dimension of  
% analysis window is higher than the signal secvence(len<N)
if nargin<3; N=1024; end 
if nargin<4; step=fix(N/2); end 
if len<N; s(len+1:N)=0; len=N; end 
if nargin<5 | len < ((iter-1)*step+N),
    iter=floor((len-N)/step);
end
B=[70, 500, 1000, 2000, 5000];
f=[0:fix(N/2)]*fs/N;
n_band = length(B)-1;
e_band = zeros(n_band, iter);
for i=1:step:iter*step
    SA=abs(fft(s(i:i+N-1) .* blackman(N))).^2;
    for j=1:n_band
        e_band(j,ceil(i/step))=sum(SA(f>B(j) & f<=B(j+1)));
    end
end
sum_band = sum(e_band);
% !!! if exists section where the signal is 0 (energy =0) !!!
sum_band = (sum_band==0)*0.000001+ (sum_band~=0).*sum_band;

for j=1:n_band
    e_band(j,:)=e_band(j,:)./sum_band;
end
%--------------------------------------------------------------------------
function [avg_s, std_s] = fuzzy_mean(s, fs, t_ms)
% Compute the average and the standard deviation associating different weights for
% the information situated at the borders (fuzzy trapezoidal window - t_ms)
if length(s)==1, avg_s=s; std_s=0; return; end
if nargin<2; t_ms=0; n=0;
else n = round(fs*t_ms/1000);
end
%in the case when a operation conduct to a NaN value (ex. division with zero)
s = s(find(~isnan(s))); 
len = length(s);
if(2*n > len), n=round(len/2); end
w = ones(1,len);  %weight values
if t_ms~=0,
    w(1:n)=([1:n]-1)/(n-1);
    w(end-n+1:end)=(n-[1:n])/(n-1);
end
avg_s = sum(s.*w)/len;
std_s = sqrt(sum(((s-avg_s).*w).^2)/(len-1));
%--------------------------------------------------------------------------
function class_id = class_name(s)
n_char = length(find(s~=0));
if n_char==0 , class_id=0; return; end %pauses between utterances
if n_char==1 & (s(1)=='%' | s(1)=='$'), class_id=0; return; end
if n_char==1 & length(find('aeiou'==s(1))), class_id=1; return; end %vowels
if n_char==2 & s(1)=='a' & (s(2)=='+' | s(2)=='-'), class_id=1; return; end %vowels 'a+','a-'
if n_char==1 & length(find('bcdfghjpstvz'==s(1))), class_id=3; return; end %consonants
if n_char==2 & (strcmp(s(1:2),'tz') | strcmp(s(1:2),'sh')), class_id=3; return; end
if n_char==1 & length(find('lmnr'==s(1))), class_id=2; return; end %semivowels
class_id=4; %diphtongs, triphtongs, ce, ci, ge, gi, che, chi,...

%dift2 = ['ai'; 'au'; 'ei'; 'eu'; 'ii'; 'iu'; 'oi'; 'ou'; 'ui'; 'ea'; 'eo'; 'ia'; 'ie'; 'io'; 'oa'; 'ua'];
%dift3 = ['a+i'; 'a+u'; 'a-i'; 'a+i'; 'ua+'];
%s2=s(1:2); for i = 2:size(dift2,1), s2=[s2; s(1:2)]; end
%if length(find(prod(double(s2==dift2),2))), disp('diftong'); end
%--------------------------------------------------------------------------
